#! /bin/sh
#
# Trident Blade3D patch release 1.00
# by ITO Takayuki <yuki@fsinet.or.jp>
# http://hp.vector.co.jp/authors/VA001240/
#
# This script modifies BeOS R5 Trident Blade3D/MVP4 driver
# for use with another Trident video chipset
#
# 1.00	first release
#
# PCI vendor ID: 1023h (Trident)
# PCI device IDs:
# Blade3D 9880     9880h
# CyberBlade i7    8420h (supported)
# CyberBlade i1    8520h
# CyberBlade e4    9540h
# Blade3D/MVP4     8400h (supported)
# Blade3D/ProMedia 8500h

SRCDIR=/system/add-ons/kernel/drivers/bin
FILE=trident

cd `dirname $0`

alert --warning "This script will apply patch to the Trident driver for use with another Trident chipset." OK Quit > /dev/null
if [ $? -eq 1 ]
then
	alert --stop "Aborted."
	exit 1
fi

if [ ! -f ./bsubst ]
then
	if [ -f bsubst.c ]
	then
		cc -o bsubst bsubst.c
	else
		alert --stop "bsubst is missing."
		exit 1
	fi
fi

# check original driver

if [ ! -f $SRCDIR/$FILE ]
then
	alert --stop "Original driver $SRCDIR/$FILE not present."
	exit 1
fi

# chipset detection

TMPFILE=/tmp/poke.log

poke > $TMPFILE <<END
pci
quit
END

# XXX: assumes all Trident devices are video devices and looks at the first device only
ID=`grep 'vendor 1023' $TMPFILE | head -1 | cut -d ' ' -f 10`
rm $TMPFILE
if [ "x$ID" = x ]
then
	alert --stop "No Trident video chipset detected."
	exit 1
fi

case $ID in
9880)	CHIP=Blade3D_9880;;
8420)	CHIP=CyberBlade_i7;;
8520)	CHIP=CyberBlade_i1;;
9540)	CHIP=CyberBlade_e4;;
8400)	CHIP=Blade3D_MVP4;;
8500)	CHIP=Blade3D_ProMedia;;
*)	CHIP=trident_unknown;;
esac
if [ $CHIP = trident_unknown ]
then
	alert --info "Unknown Trident video chipset detected. Device ID is $ID."
else
	alert --info "Trident video chipset $CHIP detected. Device ID is $ID."
fi
IDH=`echo $ID | cut -c 1-2`
IDL=`echo $ID | cut -c 3-4`
SRC="66 3D 20 84"
DST="66 3D $IDL $IDH"

# apply patch to driver

if [ -f $SRCDIR/$FILE.gz ]
then
	alert --stop "Patch is already applied."
	exit 1
fi

alert --info "Processing $SRCDIR/$FILE."
./bsubst "$SRC" "$DST" < $SRCDIR/$FILE > $SRCDIR/$FILE.new

cmp -s $SRCDIR/$FILE $SRCDIR/$FILE.new
if [ $? -eq 0 ]
then
	alert --stop "Patch is not applicable. Nothing altered."
	rm $SRCDIR/$FILE.new
	exit 1
fi

gzip -c $SRCDIR/$FILE > $SRCDIR/$FILE.gz
mv $SRCDIR/$FILE.new $SRCDIR/$FILE

alert --info "Done."
exit 0
